from zona.models import Item, Metadata, ItemType
from zona import markdown as zmd
from zona import util
from pathlib import Path
import frontmatter
from dacite import from_dict


def split_metadata(path: Path) -> tuple[Metadata, str]:
    post = frontmatter.load(str(path))
    # TODO: handle missing metadata
    meta = post.metadata
    # TODO: handle malformed metadata
    metadata = from_dict(
        data_class=Metadata,
        data=meta,
    )
    return metadata, post.content


def discover(root: Path, out_dir: Path) -> list[Item]:
    required_dirs = {"content", "templates"}
    found_dirs = {p.name for p in root.iterdir() if p.is_dir()}
    missing = required_dirs - found_dirs
    assert not missing, f"Missing required directories: {', '.join(missing)}"

    items: list[Item] = []

    subdir = "content"
    base = root / subdir
    for path in base.rglob("*"):
        if path.is_file():
            print(f"{subdir}: {path.relative_to(root)}")
            # we only parse markdown files not in static/
            destination = out_dir / path.relative_to(base)
            item = Item(
                source=path,
                destination=destination,
                url=str(destination.relative_to(out_dir)),
            )
            if path.name.endswith(".md") and not path.is_relative_to(
                root / "content" / "static"
            ):
                item.metadata, item.content = split_metadata(path)
                item.type = ItemType.MARKDOWN
                item.copy = False
                name = destination.stem
                if name == "index":
                    item.destination = item.destination.with_suffix(".html")
                else:
                    relative = path.relative_to(base).with_suffix("")
                    name = relative.stem
                    item.destination = (
                        out_dir / relative.parent / name / "index.html"
                    )
            items.append(item)
    return items


def build(items: list[Item]):
    for item in items:
        dst = item.destination
        # create parent dirs if needed
        if item.type == ItemType.MARKDOWN:
            assert item.content is not None
            # parse markdown and render as html
            raw_html = zmd.md_to_html(item.content)
            # TODO: apply to template here
            util.ensure_parents(dst)
            dst.write_text(raw_html, encoding="utf-8")
        else:
            if item.copy:
                util.copy_static_file(item.source, dst)
